/**
 * Imports
 *
 * @type {createError}
 */
 const createError = require('http-errors');
 const express = require('express');
 const path = require('path');
 const cookieParser = require('cookie-parser');
 const logger = require('morgan');
 const debug = require('debug')('app:server');
 const http = require('http');
 
 
 /**
  * Application
  */
 const app = express();
 
 /**
  * Express settings
  */
 // view engine setup
 app.set('views', path.join(__dirname, 'views'));
 app.set('view engine', 'ejs');
 
 app.use(logger('dev')); // Incoming HTTP request Logger
 app.use(express.json());
 app.use(express.urlencoded({ extended: false }));
 app.use(cookieParser());
 
 
 /**
  * Serve static files
  */
 app.use(express.static(path.join(__dirname, 'public')));
 
 
 /**
  * Serve routing
  */
 require('./app.routing')(app);
 
 
 /**
  * catch 404 and forward to error handler
  */
 app.use(function(req, res, next) {
     next(createError(404));
 });
 
 // error handler
 app.use(function(err, req, res, next) {
     // set locals, only providing error in development
     res.locals.message = err.message;
     res.locals.error = req.app.get('env') === 'development' ? err : {};
 
     // render the error page
     res.status(err.status || 500);
     res.render('error');
 });
 
 
 /**
  * Normalize a port into a number, string, or false.
  */
 
 const normalizePort = (val) => {
     const port = parseInt(val, 10);
 
     if (isNaN(port)) {
         // named pipe
         return val;
     }
 
     if (port >= 0) {
         // port number
         return port;
     }
 
     return false;
 };
 
 
 /**
  * Get port from environment and store in Express.
  */
 
 const port = normalizePort(process.env.PORT || 'false');
 app.set('port', port);
 
 
 /**
  * Create HTTP server.
  */
 const server = http.createServer(app);
 
 /**
  * Event listener for HTTP server "error" event.
  */
 
 const onError = (error) => {
     if (error.syscall !== 'listen') {
         throw error;
     }
 
     const bind = typeof port === 'string'
         ? 'Pipe ' + port
         : 'Port ' + port;
 
     // handle specific listen errors with friendly messages
     switch (error.code) {
         case 'EACCES':
             console.error(bind + ' requires elevated privileges');
             process.exit(1);
             break;
         case 'EADDRINUSE':
             console.error(bind + ' is already in use');
             process.exit(1);
             break;
         default:
             throw error;
     }
 };
 
 
 /**
  * Event listener for HTTP server "listening" event.
  */
 const onListening = () => {
     console.log("\n\n");
     console.log('<<<####################################>>>');
     console.log(`###> Application ${process.env.NAME} is listening on port:  ${process.env.PORT}`);
 };
/**
 * Listen on provided port, on all network interfaces.
 */
server.on('error', onError);
server.on('listening', onListening);
server.listen(port);
process.on('uncaughtException', err => {
    console.error(err, 'Uncaught Exception thrown');
});