/**
 ** How it works: If 'message' event listener activated calls the createChart
 */
(function () {
    // Attributes
    let selectedElement = window.selectedDataElement;
    let selectedElement2 = window.selectedDataElement;
    let myLine = {};


    // Methodes
    /**
     * * If data arrived we need to check it for the need to update the chart or not.
     */

    const updateChartDataset = (newChartData) => {
        if (myLine) {
            myLine.data.datasets[0].data = newChartData.map((datasetitem) => {
                if (datasetitem[selectedElement]) {
                    //Need to stringify to use .indexOf()
                    datasetitem[selectedElement] = datasetitem[selectedElement] + '';

                    /**
                    * * Need to replace "," to ".", because chart.js can't show the chart value with ",".
                    */
                    if (datasetitem[selectedElement].toLowerCase().indexOf(',') === -1) {
                        return Number(datasetitem[selectedElement]);
                    } else {
                        return Number(datasetitem[selectedElement].replace(',', '.'));
                    }
                }
                else {
                    return null;
                }
            });
            myLine.update();
        }
    };

    const updateChartDataset2 = (newChartData) => {
        if (myLine) {
            myLine.data.datasets[1].data = newChartData.map((datasetitem) => {
                if (datasetitem[selectedElement2]) {
                    //Need to stringify to use .indexOf()
                    datasetitem[selectedElement2] = datasetitem[selectedElement2] + '';

                    /**
                    * * Need to replace "," to ".", because chart.js can't show the chart value with ",".
                    */
                    if (datasetitem[selectedElement2].toLowerCase().indexOf(',') === -1) {
                        return Number(datasetitem[selectedElement2]);
                    } else {
                        return Number(datasetitem[selectedElement2].replace(',', '.'));
                    }
                }
                else {
                    return null;
                }
            });
            myLine.update();
        }
    };

    /**
     * * Load chart dataset 1 - 2
     */
    const initChartDataset = (customProperties, queryParams) => {
        return {
            label: queryParams.dtsname, // Your chart name
            data: customProperties.dataset.map((datasetitem) => {
                if (datasetitem[selectedElement] || datasetitem[selectedElement] === 0) {
                    datasetitem[selectedElement] = datasetitem[selectedElement] + '';
                    if (datasetitem[selectedElement].toLowerCase().indexOf(',') === -1) {
                        return String(datasetitem[selectedElement]);
                    } else {
                        return String(datasetitem[selectedElement].replace(',', '.'));
                    }
                }
                else {
                    return null;
                }
            }),
            backgroundColor: queryParams.customColors
                ? customProperties.datasetConfig.colors
                : [queryParams.backgroundColor],
            borderColor: [queryParams.borderColor],
            borderWidth: queryParams.borderWidth,
            fill: 'start', //fill the chart under the line
            cubicInterpolationMode: 'monotone', //smooth lines
            tension: 0.4,
            spanGaps: queryParams.spanGaps, // drawn between points with no or null data.
        }
    }

    const initChartDataset2 = (customProperties, queryParams) => {
        if (queryParams.showTwoLine) {
            return {
                label: queryParams.dtsname2, // Your chart name
                data: customProperties.dataset2.map((datasetitem) => {
                    if (datasetitem[selectedElement2] || datasetitem[selectedElement2] === 0) {
                        datasetitem[selectedElement2] = datasetitem[selectedElement2] + '';
                        if (datasetitem[selectedElement2].toLowerCase().indexOf(',') === -1) {
                            return String(datasetitem[selectedElement2]);
                        } else {
                            return String(datasetitem[selectedElement2].replace(',', '.'));
                        }
                    }
                    else {
                        return null;
                    }
                }),
                type: queryParams.chartType2,
                backgroundColor: queryParams.customColors
                    ? customProperties.datasetConfig.colors
                    : [queryParams.backgroundColor2],
                borderColor: [queryParams.borderColor2],
                borderWidth: queryParams.borderWidth2,
                fill: 'start', //fill the chart under the line
                cubicInterpolationMode: 'monotone', //smooth lines
                tension: 0.4,
                spanGaps: queryParams.spanGaps, // drawn between points with no or null data.
            }
        } else {
            return {};
        }
    }

    /**
     * * Load chart options
     */
    const initChartOptions = (queryParams) => {
        return {
            responsive: true,
            maintainAspectRatio: false,
            scales: {
                x: {
                    ticks: {
                        display: queryParams.displayXlabels,
                        fontSize: queryParams.fontSize,
                        minRotation: queryParams.labelsRotation, //Change X labels rotation in °
                    },
                },
                y: {
                    min: queryParams.minYValue,
                    ticks: {
                        display: queryParams.displayYlabels,
                    },
                },
            },
            plugins: {
                legend: {
                    position: queryParams.legendPosition,
                    display: queryParams.showLegend,
                    title: {
                        display: true,
                        text: queryParams.title,
                    },
                },
            },
        }
    }

    /**
     * * Load chart configuration
     */
    const initConfig = (customProperties, queryParams) => {
        return {
            type: queryParams.chartType, //The selected chart type.
            data: {
                labels: customProperties.dataset.map((datasetitem) => {
                    return datasetitem[customProperties.datasetConfig.selectedLabelsElement]; //Fill labels
                }),
                datasets: [
                    initChartDataset(customProperties, queryParams),
                    initChartDataset2(customProperties, queryParams),
                ],
            },
            options: initChartOptions(queryParams),
        };
    }

    /**
     * * Remove null data points from the data
     */
    const removeNulls = (config) => {
        for (let j = 0; j < 2; j++) {
            let len = config.data.datasets[j].data.length;
            for (let i = 0; i < len;) {
                if (config.data.datasets[j].data[i] === null) {
                    config.data.datasets[j].data.splice(i, 1);
                    config.data.labels.splice(i, 1);
                } else if (config.data.datasets[j].data.length < 2) {
                    break;
                } else {
                    i++;
                }
            }
        }
    }

    /**
     * * Set chart opacity (0-100)
     */
    const setChartOpacity = (queryParams) => {
        document.querySelector('#chart-area').style.opacity = queryParams.opacity / 100; //set opacity
    }

    const createChart = (event) => {
        const messageType = event.data.messageType;
        const changedProperty = event.data.changedProperty;
        const customProperties = event.data.customProperties;
        const queryParams = event.data.queryParams;
        // Here we will call the updateChartDataset function and set the chart configuration.
        if (
            messageType === 'customWidget_boundDataChanged' &&
            (changedProperty === 'dataset' || changedProperty === 'dataset2')
        ) {
            if (changedProperty === 'dataset') {
                updateChartDataset(event.data.newValue);
            }
            else if (changedProperty === 'dataset2') {
                updateChartDataset2(event.data.newValue);
            }
        } else if (messageType === 'customWidget_sendCustomProperties') {
            selectedElement = customProperties.datasetConfig.selectedDataElement;
            selectedElement2 = customProperties.datasetConfig2.selectedDataElement;
            /**
             ** Here you can manage your chart configuration 
             ** You can find more about the chart configuration on the chart.js website (https://www.chartjs.org/docs/latest/configuration/)
            */
            const config = initConfig(customProperties, queryParams);

            /**
             * *The chart was created here.
            */
            const ctx = document.getElementById('chart-area').getContext('2d');
            Chart.defaults.font.size = queryParams.fontSize; //font size change for the chart
            myLine = new Chart(ctx, config);

            /**
             * * We check here that the dataset has null data and if it has we .splice(i,1) it.
             * * config is a reference
            */
            removeNulls(config);

            /**
            * * Update lines
            */
            myLine.update();

            /**
             * * Set opacity
             */
            setChartOpacity(queryParams);
        }
    }

    // Listeners
    window.addEventListener('message', createChart);

    window.onload = () => {
        window.parent.window.postMessage({ messageType: 'customWidget_requestCustomProperties' }, '*');
        console.log('Called, customWidget_requestCustomProperties');
    };
}());